/*
 * de.superx.etl - a package for controlling ETL routines
 * Copyright (C) 2021 Daniel Quathamer <danielq@memtext.de>
 *
 * This package is licensed under the CampusSource License; 
 * http://www.campussource.de/org/license/
 */
package de.superx.elt;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.io.StringWriter;
import java.io.UnsupportedEncodingException;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.Properties;
import de.memtext.util.PropUtils;

import javax.sql.DataSource;

import de.superx.bin.SxConnection;
import de.superx.common.Sichten;
import de.superx.etl.EtlUtils;

public class EtlAction {
	protected Properties params;
	private boolean contOnError;
	private String targetTable;
	private String logoutput;
	protected StringWriter actionOutput=new StringWriter();
	protected StringWriter actionLog=new StringWriter();
	
	private String loglevel="INFO";
	private long starttime = new java.util.Date().getTime() ;
	protected long endtime ;
	private int returnCode;
	private Connection dbconnection;
	private Properties env;
	private String uniquename;
	protected String logfile;	
	protected String propfile;
	private String encoding="UTF-8";
	protected DataSource dataSource;


	public EtlAction(String uniquename, String logfile, String propfile) {
		this.uniquename = uniquename;
		this.logfile = logfile;
		this.propfile=propfile;
		this.addActionLog("Action initialized"+EtlUtils.NEWLINE);

	}
	public EtlAction(String uniquename) {
		this.uniquename = uniquename;
		this.addActionLog("Action initialized"+EtlUtils.NEWLINE);

	}	
	public DataSource getDataSource() {
		return dataSource;
	}
	public void setDataSource(DataSource dataSource) {
		this.dataSource = dataSource;
	}
	public Connection getDbconnection() {
		return dbconnection;
	}
	public void setDbconnection(Connection dbconnection) {
		this.dbconnection = dbconnection;
	}

	public String getUniquename() {
		return uniquename;
	}
	public void setUniquename(String uniquename) {
		this.uniquename = uniquename;
	}
	public String getLogfile() {
		return logfile;
	}
	public void setLogfile(String logfile) {
		this.logfile = logfile;	
	}
	public Properties getEnv() {
		return env;
	}
	public void setEnv(Properties env) {
		this.env = env;
	}
	public Properties getParams() {
		return params;
	}
	public void setParams(Properties params) {
		this.params = params;
	}

	public boolean isContOnError() {
		return contOnError;
	}
	public void setContOnError(boolean contOnError) {
		this.contOnError = contOnError;
	}
	public String getTargetTable() {
		return targetTable;
	}
	public void setTargetTable(String targetTable) {
		this.targetTable = targetTable;
	}
	public String getLogoutput() {
		return logoutput;
	}
	protected void setLogoutput(String log) {
		this.logoutput = log;
	}
	protected void appendLog(String log) {
		this.logoutput+= log;
	}
	public String getLoglevel() {
		return loglevel;
	}
	public void setLoglevel(String loglevel) {
		this.loglevel = loglevel;
	}
	public long getStarttime() {
		return starttime;
	}
	public void setStarttime(long starttime) {
		this.starttime = starttime;
	}
	public long getEndtime() {
		return endtime;
	}
	public void setEndtime(long endtime) {
		this.endtime = endtime;
	}
	public long getDuration() {
		return (this.endtime - this.starttime)/1000;
	}


	public String getPropfile() {
		return propfile;
	}

	public void setPropfile(String propfile) {
		this.propfile = propfile;
	}

	public int getReturnCode() {
		return returnCode;
	}
	public StringWriter getActionOutput() {
		return actionOutput;
	}
	public void setActionOutput(StringWriter actionOutput) {
		this.actionOutput = actionOutput;
	}
	public StringWriter getActionLog() {
		return actionLog;
	}
	
	public void setActionLog(StringWriter actionLog) {
		this.actionLog = actionLog;
	}
	public void addActionLog(String actionLog) {
		
		String log=de.memtext.util.DateUtils.getTodayString()+ " " + de.memtext.util.DateUtils.getNowString();
		this.actionLog.append(log+"- "+ actionLog+EtlUtils.NEWLINE);
	}
	protected void setReturnCode(int returnCode) {
		this.returnCode = returnCode;
	}
	public int execute() throws SQLException, Exception
	{
		int returncode=0;
		endtime=new java.util.Date().getTime() ;
		return returncode;
	}

	public Connection getConnection(Connection myConnection) throws Exception {
		String myDefaultPropFile=de.superx.elt.EtlUtils.WEBINFDIR+de.superx.elt.EtlUtils.PATHSEP+"db.properties";
		if(myConnection==null)
		{
			if(propfile==null)
				propfile=myDefaultPropFile;
			SxConnection mySxConnection = null;
			mySxConnection = new SxConnection();
			mySxConnection.setPropfile(propfile);

			myConnection = mySxConnection.getConnection();


		}
		return myConnection;
	}
}
