/*
 * de.superx.etl - a package for controlling ETL routines
 * Copyright (C) 2021 Daniel Quathamer <danielq@memtext.de>
 *
 * This package is licensed under the CampusSource License; 
 * http://www.campussource.de/org/license/
 */
package de.superx.elt;


import java.io.InputStream;
import java.io.PrintStream;
import java.io.StringWriter;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Properties;
import java.util.logging.Logger;

import javax.sql.DataSource;

import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowCallbackHandler;
import org.springframework.jdbc.datasource.DriverManagerDataSource;

import de.memtext.rights.RightsKeyEntry;
import de.memtext.util.GetOpts;
import de.superx.bin.Doquery;
import de.superx.bin.SxConnection;
import de.superx.common.SuperX_el;
import de.superx.common.SxResultRow;
import de.superx.common.SxResultSet;
import de.superx.common.SxSqlHelper;
import de.superx.elt.ActionHandler.*;
import de.superx.servlet.SxPools;
import de.superx.util.SqlStringUtils;
import freemarker.template.TemplateBooleanModel;

public class EtlStep extends EtlAction {
	private String mandantenID;
	private Integer stepTid;
	private String stepUniquename;
	private String stepLogfile;
	private String stepCaption;
	private Integer stepSortnr;
	private Integer stepForceContinue;
	private Integer stepParentStepId;
	private String stepTypeUniquename;
	private String stepTypeCaption;
	private String stepTypeHandler;
	private Properties jobRunTimeParams;
	private Properties stepProperties;
	private SxConnection stepSxConnection;
	private Connection stepConnection;
	public EtlStep(String uniquename, String logfile, String propfile) {
		super(uniquename, logfile, propfile);
		//this.setLogoutput("Step "+uniquename);
		super.addActionLog("Step "+uniquename +" loaded");
		
	}
	public StringWriter getOutputStringWriter() {
		return sw;
	}


	public void setOutputStringWriter(StringWriter sw) {
		this.sw = sw;
	}
	private StringWriter sw;



	public void initStep(String job,Properties runTimeParams, Integer tid,
			String uniquename,
			String logfile,
			String caption,
			Integer sortnr,
			Integer forceContinue,
			Integer parentStepId,
			String typeUniquename,
			String typeCaption,
			String typeHandler
			) throws Exception
	{
		jobRunTimeParams=runTimeParams;
		stepTypeHandler=typeHandler;
		stepTid=tid;
		stepUniquename=uniquename;
		stepLogfile=logfile;
		stepCaption=caption;
		stepSortnr=sortnr;
		stepForceContinue=forceContinue;
		stepParentStepId=parentStepId;
		stepTypeUniquename=typeUniquename;
		stepTypeCaption=typeCaption;
		this.setLogfile(logfile);
		super.addActionLog("Initializing Step: "+caption);
		super.addActionLog("Handler: "+typeHandler);
		//this.appendLog("Params: "+runTimeParams.toString());
		//get Params:
		Connection myConnection=this.getDbconnection();
		if(myConnection==null)
			myConnection=this.getConnection(myConnection);
		this.setDbconnection(myConnection);
		if(!SqlStringUtils.checkValidKeyEntry(job))
			throw new SQLException("Invalid job");
		String sql = "select tid,prop_name,prop_value from etl_step_property where etl_step_id="+tid.toString()+";";

		SuperX_el el = new SuperX_el();
		SxSqlHelper sh=new SxSqlHelper();
		sh.execute(sql, myConnection, el);
		if (el.getError_String() != null
				&& !el.getError_String().trim().equals(""))
			throw new SQLException("\nProblem bei Step DETAILS:" + "\n\n  Meldung:"
					+ el.getError_String() + "\n  sql:" + sql);
		SxResultSet result=  el.getResultSet();
		stepProperties=null;
		stepProperties=new Properties();
		int rownr=0;
		for (Iterator it = result.iterator(); it.hasNext();) {
			rownr++;
			SxResultRow row = (SxResultRow) it.next();
			String paramName=row.get(1).toString().trim();
			String paramValue=row.get(2).toString().trim();
			String paramValueParsed=parseParams(paramValue);
			stepProperties.setProperty(paramName, paramValueParsed);
			super.addActionLog("Parameter "+paramName+":"+paramValueParsed);

		}

	}
	private String parseParams(String paramValue)
	{
		String parsedParam=paramValue;
		Enumeration runTimeParamNames = jobRunTimeParams.propertyNames();
		while (runTimeParamNames.hasMoreElements()) {
			String runTimeParamName = (String)runTimeParamNames.nextElement();
			String runTimeParamValue = jobRunTimeParams.getProperty(runTimeParamName);
			parsedParam=de.memtext.util.StringUtils.replace(parsedParam,"$"+runTimeParamName, runTimeParamValue);

		}
		return parsedParam;

	}

	public int execute()
			throws Exception
	{
		int returnCode=0;
		long numberOfRows=0;
		Logger logger =(Logger) Logger.getLogger(EtlStep.class.toString());
		
		super.addActionLog("Executing Step "+this.stepUniquename);
		Class handlerClass = Class.forName(stepTypeHandler);
		Object handlerObject = handlerClass.newInstance();
		((EtlActionHandlerI) handlerObject).setLogger(logger);
		((EtlActionHandlerI) handlerObject).setHandlerSpecificProperties(stepProperties);
		((EtlActionHandlerI) handlerObject).setRunTimeParams(jobRunTimeParams);
		((EtlActionHandlerI) handlerObject).setOutputStringWriter(this.getOutputStringWriter());
		((EtlActionHandlerI) handlerObject).setPropFile(this.getPropfile());
		
		try {
			returnCode=((EtlActionHandlerI) handlerObject).execute(this.getOutputStringWriter(),"default",this.stepUniquename,"xml");
		} catch (Exception e) {
			returnCode=1;
			super.addActionLog("Fehler beim Step "+ this.stepUniquename+ ": "+e.toString());
			//TODO forceContinue auswerten
		}
		this.setOutputStringWriter(((EtlActionHandlerI) handlerObject).getOutputStringWriter());
		
		super.addActionLog(((EtlActionHandlerI) handlerObject).getLogOutput());
		
		
		/*
		if(stepTypeHandler.equals("de.superx.bin.UnloadRecords"))
		{
			getConnection(logger, this.getPropfile());
			String query=stepParams.getProperty("select_stmt");

			sw=this.getSw();
			QueryResultSerializer myQueryResultSerializer=new QueryResultSerializer("default", stepSxConnection,query,sw);
			myQueryResultSerializer.setLogger(logger);
			NumberOfRows=myQueryResultSerializer.unloadQueryResults(this.stepUniquename,"xml","",true);
			this.setReturnCode(myQueryResultSerializer.getReturnCode());

			sw=myQueryResultSerializer.getOutputString();
			this.setSw(sw);
			//System.out.println(sw.toString());
			stepSxConnection.close();
		}
		if(stepTypeHandler.equals("de.superx.bin.Doquery"))
				
			{
				String query=stepParams.getProperty("select_stmt");
				getConnection(logger, this.getPropfile());
				SqlExecutor mySqlExecutor=new SqlExecutor("default", stepSxConnection,query,jobRunTimeParams);
				returnCode=mySqlExecutor.executeQueries();
				
				stepSxConnection.close();
			}
			
		if(stepTypeHandler.equals("de.superx.bin.UploadRecords"))
			
		{
			String target_table=stepParams.getProperty("target_table");
			String inputfile=stepParams.getProperty("path_to_inputfile");
			String xml_search_path=stepParams.getProperty("search_path");
			long numberOfRows=0;
			//getConnection(logger, this.getPropfile());
			TableUploader myUploader=new TableUploader();
			myUploader.setDbpropfile(this.getPropfile());
			myUploader.setInFormat("xml");
			myUploader.setTargetTable(target_table);
			myUploader.setSrcFile(inputfile);
			myUploader.setXml_search_path(xml_search_path);
			//myUploader.setUploadConnection(stepSxConnection.getConnection());
			myUploader.setUploadConnection(myUploader.getConnection(null,this.getPropfile()));
			//SqlExecutor mySqlExecutor=new SqlExecutor("default", stepSxConnection,query);
			numberOfRows=myUploader.uploadFile();
			this.appendLog("Number of rows loaded: "+numberOfRows);
			
			
		}
		if(stepTypeHandler.trim().equals("msg"))
			
		{
			String msg=stepParams.getProperty("msg");
			
			this.appendLog(msg);
		}*/
		//logger.info(this.getLogoutput());
		return  returnCode;
	}

}
