/*
 * de.superx.etl - a package for controlling ETL routines
 * Copyright (C) 2021 Daniel Quathamer <danielq@memtext.de>
 *
 * This package is licensed under the CampusSource License; 
 * http://www.campussource.de/org/license/
 */
package de.superx.elt;
import java.io.BufferedOutputStream;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintStream;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.JDBCType;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.PreparedStatement; 
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Types;
import java.util.logging.LogManager;
import java.util.logging.Logger;
import java.util.logging.Level;

import de.memtext.util.StringUtils;
import de.superx.bin.SxConnection;
import de.superx.bin.SxDBUtils;
import de.superx.bin.SxJdbcClient;

import java.util.Iterator;
import java.util.StringTokenizer;

import de.superx.common.SxResultSet;
import de.superx.elt.bin.SxTransformer;
import de.superx.servlet.ServletUtils;
import de.superx.servlet.SxSQL_Server;
import de.superx.util.SqlStringUtils;
/**
 * @author Daniel Quathamer Projektgruppe SuperX
 * doquery.java
 * @
 * Dieses Javaprogramm führt einen SQL-Ausdruck aus und gibt das Ergebnis aus.<br>
 * Gebrauch:<br> java doquery <Pfad zu logger-properties> <pfad zu db.properties> <sql-Ausdruck> <Ausgabeformat (txt | html | xml)>(optional) <delimiter> <mit Spaltenüberschriften (true | false)>(optional) <Ausgabedatei>(optional)
 *

 */
/*
 * SQL-Abfragen Ergebnis serialisieren nach CSV / XML
 */
public class QueryResultSerializer {
	public QueryResultSerializer(String mandantenId,SxConnection mySxConnection, String query,StringWriter sw) {
		super();
		this.mySxConnection = mySxConnection;
		this.query = prepareQuery(query);
		this.mandantenId = mandantenId;
		this.logger = Logger.getLogger("superx_" + mandantenId);
		this.outputString=sw;
	}
	public QueryResultSerializer(String mandantenId,SxConnection mySxConnection, String query,FileWriter fw) {
		super();
		this.mySxConnection = mySxConnection;
		this.query = prepareQuery(query);
		this.mandantenId = mandantenId;
		this.logger = Logger.getLogger("superx_" + mandantenId);
		this.outputFile=fw;
	}	
	public QueryResultSerializer(String mandantenId,SxConnection mySxConnection, ResultSet rs,FileWriter fw) {
		super();
		this.mySxConnection = mySxConnection;
		this.myrs=rs;
		this.mandantenId = mandantenId;
		this.logger = Logger.getLogger("superx_" + mandantenId);
		this.outputFile=fw;
	}	
	public QueryResultSerializer(String mandantenId,SxConnection mySxConnection, ResultSet rs,StringWriter sw) {
		super();
		this.mySxConnection = mySxConnection;
		this.myrs=rs;
		this.mandantenId = mandantenId;
		this.logger = Logger.getLogger("superx_" + mandantenId);
		this.outputString=sw;
	}
	public final String NEWLINE=System.getProperty("line.separator");
	private String outfile;
	private SxConnection mySxConnection;
	private String query;
	private String CSVdelim;
	private Boolean printColNames;
	private String outFormat;
	private Logger logger;
	private String mandantenId="default";
	private Statement st;
	private Connection myConnection;
	private String[] colnames;
	private int[] coltypes;
	private int columnCount;
	private StringBuffer outString=new StringBuffer("");
	//private PrintStream stringOutWriter=new PrintStream(outString); 
	//private PrintWriter output=new PrintWriter(stringOutWriter);
	private String outputHeader="";
	private String outputFooter="";
	private BufferedOutputStream outputStream;
	public StringWriter outputString;
	public FileWriter outputFile;
	private int returnCode;
	private ResultSet myrs;

	public String getOutfile() {
		return outfile;
	}
	public void setOutfile(String outfile) {
		this.outfile = outfile;
	}
	public SxConnection getMySxConnection() {
		return mySxConnection;
	}
	public void setMyConnection(SxConnection mySxConnection) {
		this.mySxConnection = mySxConnection;
	}
	public String getQuery() {
		return query;
	}
	public void setQuery(String query) {
		this.query = query;

	}

	public String getOutFormat() {
		return outFormat;
	}
	public void setOutFormat(String outFormat) {
		this.outFormat = outFormat;
	}
	public Logger getLogger() {
		return logger;
	}
	public void setLogger(Logger logger) {
		this.logger = logger;
	}
	public String getMandantenId() {
		return mandantenId;
	}
	public void setMandantenId(String mandantenId) {
		this.mandantenId = mandantenId;
	}
	public String[] getColnames() {
		return colnames;
	}
	public void setColnames(String[] colnames) {
		this.colnames = colnames;
	}
	public int[] getColtypes() {
		return coltypes;
	}
	public void setColtypes(int[] coltypes) {
		this.coltypes = coltypes;
	}
	public StringWriter getOutputString() {
		return outputString;
	}
	public void setOutputString(StringWriter output) {
		this.outputString = output;
	}
	public FileWriter getOutputFile() {
		return outputFile;
	}
	public void setOutputFile(FileWriter outputFile) {
		this.outputFile = outputFile;
	}
	public int getColumnCount() {
		return columnCount;
	}
	public void setColumnCount(int columnCount) {
		this.columnCount = columnCount;
	}



	public int getReturnCode() {
		return returnCode;
	}
	public void setReturnCode(int returnCode) {
		this.returnCode = returnCode;
	}
	protected ResultSet getMyrs() {
		return myrs;
	}
	protected void setMyrs(ResultSet myrs) {
		this.myrs = myrs;
	}
	public long unloadQueryResults(String uniquename, String outformat,String delim,boolean printColname) throws Exception
	{
		returnCode=0;
		long numberOfRows=0;
		String zs="";
		if(myrs==null)
		{
		SxResultSet rs = null;

		SxJdbcClient myClient = new SxJdbcClient(logger, mySxConnection.getPropfile(), query);


		//logger.info("Executing sql: " + query);
		myClient.Rs_executeALL();
		logger.info("Getting resultset") ;
		myrs = myClient.getRs();
		//logger.info("Starting output for: " + query);
		}
		if(myrs != null ) 
		{
			ResultSetMetaData rsmd = myrs.getMetaData();
			setColnamesAndTypes(rsmd);

		
		if(outformat.equalsIgnoreCase("xml"))
		{
			zs+="<unload name=\""+uniquename+"\"><query><![CDATA["+query+"]]></query><rs>";
			if(printColname)
				zs+=this.getColnamesXML();
		}
		else
		{
			//CSV:
			if(printColname)
				zs+=this.getColnamesCSV(delim)+"\n";
		}	
		//this.outputStream.w
		//StringWriter sw = new StringWriter();
		//sw.write(zs);
		//this.setOutputString(sw);

		try {
			numberOfRows=printResult(outformat,zs,myrs,delim);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			returnCode=1;
		}
		}
		return numberOfRows;
	}
	private long printResult(String outformat,String header,ResultSet outrs,String delim) throws SQLException, IOException
	{
		StringBuffer line = new StringBuffer();
		int types[]=this.getColtypes();
		String lbl[]=this.getColnames();
		String f_wert=null;
		Object o = null;
		int rowCounterBeforeFlush = 0;
		long rowCounter = 0;
		FileWriter fw = null;
		StringWriter sw=null;
		String rowDelim1=(outformat.equalsIgnoreCase("xml"))?"<row>":"";
		String rowDelim2=(outformat.equalsIgnoreCase("xml"))?"</row>"+NEWLINE:"\n"; //bei CSV ist Zeilentrenner immer \n, nicht DOS-Anpassung
		String footer=(outformat.equalsIgnoreCase("xml"))?"</rs></unload>":"";

		boolean writeFileOutput=(this.outputFile==null)?false:true;
		int colNumber=this.getColumnCount();
		if(writeFileOutput)
		{
			fw=this.getOutputFile();
			fw.write(header);
		}
		else
		{
			sw=this.getOutputString();
			sw.write(header);
		}
		while (outrs.next())
		{
			line.setLength(0);
			line.append(rowDelim1);
			for (int i = 1; i <= colNumber; i++) {
				if (types[i-1] == Types.LONGVARCHAR) {
					//Der Informix-Treiber verlangt hier getString
					// statt getObject!
					o = outrs.getString(i);
				} else {
					o = outrs.getObject(i);
				}
				//vergl. de.superx.common.Maske.getCSV
				f_wert = SxDBUtils.field_value(o);
				f_wert = prepareFieldValue(outformat,lbl[i-1],f_wert,delim);

				if (i < colNumber)
					line.append( f_wert );
				else {
					line.append(f_wert );
					line.append(rowDelim2);
				}

			}

			if(writeFileOutput)
			{
				//fw=this.getOutputFile();

				fw.write(line.toString());
				rowCounterBeforeFlush++;
				if (rowCounterBeforeFlush > 10000) {
					de.memtext.util.MemoryUtils.printfree();
					fw.flush();

					rowCounterBeforeFlush = 0;
				}
			}
			else
			{
				//sw=this.getOutputString();
				sw.write(line.toString());
			}
			rowCounter++;


		} ; //von while
		outrs.close();

		if(writeFileOutput)
		{
			if(outformat.equalsIgnoreCase("xml"))
				fw.write(footer);
			fw.flush();
			fw.close();
		}
		else
		{
			if(outformat.equalsIgnoreCase("xml"))
				sw.write(footer);
			sw.close();
			this.setOutputString(sw);
		}
		return rowCounter;






	}
	private void setColnamesAndTypes(ResultSetMetaData rsmd) throws SQLException
	{
		int cols = rsmd.getColumnCount();
		int[] types = new int[cols];
		String[] lbl = new String[cols];

		// Print the result column names?
		for (int i = 0; i < cols; i++) {
			lbl[i] = rsmd.getColumnLabel(i+1);
			types[i] = rsmd.getColumnType(i+1);
		}

		this.setColnames(lbl);
		this.setColtypes(types);
		this.setColumnCount(cols);      
	}

	

	private String getColnamesCSV(String delim) {
		String headZs = "";
		int colNumber = this.getColnames().length;
		for (int i = 0; i < colNumber; i++) {
			headZs += ( this.getColnames()[i] +delim);
		}

		return headZs;
	}
	private String getColnamesXML() {
		String headZs = "<rsmd>";
		int colNumber = this.getColnames().length;
		for (int i = 0; i < colNumber; i++) {
			headZs += "<fld name=\"" + this.getColnames()[i] + "\" type=\""+SxDBUtils.getSqlTypeName(this.getColtypes()[i])+"\"> </fld>"+NEWLINE;

		}
		headZs+="</rsmd>";

		return headZs;
	}
	private String prepareQuery(String query)
	{
		query = query.trim();
		//DOS produces unnecessary ".." around the stmt
		if (query.length() > 0) {

			if (query.startsWith("\""))
				query = query.substring(1, query.length());
			if (query.endsWith("\""))
				query = query.substring(0, query.length() - 1);
		}
		return query;
	}
	private String prepareFieldValue(String outformat,String fieldName, String f_wert,String delim)
	{
		String fieldValue="";
		if (outformat.equals("xml"))
		{
			fieldValue+="<fld name=\"" + fieldName + "\">";
			f_wert=encodeCDATA(f_wert);
			if(f_wert.indexOf("&") > -1 || f_wert.indexOf("<") > -1 || f_wert.indexOf(">") > -1)
				fieldValue += "<![CDATA[" + f_wert + "]]>";
			else
				fieldValue +=  f_wert;
			fieldValue+="</fld>"+NEWLINE;
		}
		//wenn der Feldwert zufällig das Trennzeichen enthält, wird es mit "\" maskiert
		if (outformat.equals("txt")) {
			fieldValue = SxDBUtils.prepareInformixCsv(f_wert);
			if (fieldValue != null && (fieldValue.indexOf(delim) > -1)) fieldValue = de.memtext.util.StringUtils.replace(fieldValue, delim, "\\" + delim);
			fieldValue+=delim;
		}


		return fieldValue;
	}
	/**
	 * Beim XML Export darf der Feldinhalt nicht CDATA-Tags enthalten, dies wrürde beim Import
	 * Probleme machen. Daher durch Platzhalter ersetzen
	 * 
	 * @param Feldinhalt
	 *            PG/IDS
	 * @return Feldinhalt (ersetzt)
	 */
	private String encodeCDATA(String x)
	{
		if (x==null) x="";
		x = StringUtils.replace(x,"<![CDATA[",
				"CDATASTART" );
		x = StringUtils.replace(x, "]]>",
				"CDATAEND");
		return x;

	}
}
