/*
 * de.superx.etl - a package for controlling ETL routines
 * Copyright (C) 2021 Daniel Quathamer <danielq@memtext.de>
 *
 * This package is licensed under the CampusSource License; 
 * http://www.campussource.de/org/license/
 */
package de.superx.elt;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.JDBCType;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.PreparedStatement; 
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Types;
import java.util.logging.LogManager;
import java.util.logging.Logger;
import java.util.logging.Level;

import de.memtext.util.DateUtils;
import de.memtext.util.StringUtils;
import de.superx.bin.FMParser;
import de.superx.bin.SxConnection;
import de.superx.bin.SxDBUtils;
import de.superx.bin.SxJdbcClient;

import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Properties;
import java.util.StringTokenizer;

import de.superx.bin.XUpdater;
import de.superx.common.FieldContainer;
import de.superx.elt.bin.SxTransformer;
import de.superx.servlet.ServletUtils;
import de.superx.servlet.SuperXManager;
import de.superx.servlet.SxPools;
import de.superx.servlet.SxSQL_Server;
import de.superx.util.SqlStringUtils;
/**
 * @author Daniel Quathamer Projektgruppe SuperX
 * doquery.java
 * @
 * Dieses Javaprogramm führt einen SQL-Ausdruck aus und gibt das Ergebnis aus.<br>
 * Gebrauch:<br> java doquery <Pfad zu logger-properties> <pfad zu db.properties> <sql-Ausdruck> <Ausgabeformat (txt | html | xml)>(optional) <delimiter> <mit Spaltenüberschriften (true | false)>(optional) <Ausgabedatei>(optional)
 *

 */
/*
 * SQL-Abfragen Ergebnis serialisieren nach CSV / XML
 */
public class SqlExecutor {
	public SqlExecutor(String mandantenId,SxConnection mySxConnection, String sqlScript, Properties params) {
		super();
		this.mySxConnection = mySxConnection;
		this.sqlScript = sqlScript;
		this.mandantenId = mandantenId;
		this.logger = Logger.getLogger("superx_" + mandantenId);
		this.params=params;
		
	}
	public SqlExecutor(String mandantenId,SxConnection mySxConnection, File sqlScriptFile, Properties params) throws IOException {
		super();
		this.mySxConnection = mySxConnection;
		this.mandantenId = mandantenId;
		this.sqlScriptFile=sqlScriptFile;
		this.logger = Logger.getLogger("superx_" + mandantenId);
		this.params=params;
		
		if (!sqlScriptFile.exists()) {
			throw new IOException("Datei nicht gefunden: " + sqlScriptFile.getAbsolutePath());
		}
		BufferedReader in;
		//--- InputStream generieren ---//
		in = new BufferedReader(new InputStreamReader(new FileInputStream(sqlScriptFile)));

		//--- Verarbeiten der Datei ---//
		String sql = "";
		String text = "";
		while ((text = in.readLine()) != null) {
		    //MB auskommentiert 27.4.05
		    //löschte ( in
		    //insert into
		    //(
		    //tid,...
			//if (text.length() > 1)
		  	sql += NEWLINE + text.trim();
		}
		in.close();
		this.sqlScript =sql;
		
	}	
	public final String NEWLINE=System.getProperty("line.separator");
	private SxConnection mySxConnection;
	private String sqlScript;
	private File sqlScriptFile;
	private Logger logger;
	private String mandantenId="default";
	private Statement st;
	private Connection myConnection;
	private StringBuffer outString=new StringBuffer("");
	//private PrintStream stringOutWriter=new PrintStream(outString); 
	//private PrintWriter output=new PrintWriter(stringOutWriter);
	private int returnCode;
	private String outfile="";
	private String CSVdelim="^";
	private boolean printColNames=false;
	private String outFormat="txt";
	private long numberOfRows;

	private Properties params;


	public SxConnection getMySxConnection() {
		return mySxConnection;
	}
	public void setMyConnection(SxConnection mySxConnection) {
		this.mySxConnection = mySxConnection;
	}
	public String getSqlScript() {
		return sqlScript;
	}
	public void setSqlScript(String sqlScript) {
		this.sqlScript = sqlScript;

	}

	public Logger getLogger() {
		return logger;
	}
	public void setLogger(Logger logger) {
		this.logger = logger;
	}
	public String getMandantenId() {
		return mandantenId;
	}
	public void setMandantenId(String mandantenId) {
		this.mandantenId = mandantenId;
	}

	public String getOutfile() {
		return outfile;
	}
	public void setOutfile(String outfile) {
		this.outfile = outfile;
	}
	public String getCSVdelim() {
		return CSVdelim;
	}
	public void setCSVdelim(String cSVdelim) {
		CSVdelim = cSVdelim;
	}
	public boolean getPrintColNames() {
		return printColNames;
	}
	public void setPrintColNames(boolean printColNames) {
		this.printColNames = printColNames;
	}
	public String getOutFormat() {
		return outFormat;
	}
	public void setOutFormat(String outFormat) {
		this.outFormat = outFormat;
	}
	public int getReturnCode() {
		return returnCode;
	}
	public void setReturnCode(int returnCode) {
		this.returnCode = returnCode;
	}
	public long getNumberOfRows() {
		return numberOfRows;
	}
	public void setNumberOfRows(long numberOfRows) {
		this.numberOfRows = numberOfRows;
	}
	public Properties getParams() {
		return params;
	}
	public void setParams(Properties params) {
		this.params = params;
	}
	public StringBuffer getOutString() {
		return outString;
	}
	public void setOutString(StringBuffer outString) {
		this.outString = outString;
	}
	public int executeQueries() throws Exception
	{
		returnCode=0;
		FileWriter fw;
		StringWriter sw;
		SxTransformer myTransformer;
		QueryResultSerializer myQueryResultSerializer=null;
		if (sqlScript.toLowerCase().indexOf("<xupdate>")>-1)
		{
			SxConnection myConnection = new SxConnection();
			myConnection.setPropfile(mySxConnection.getPropfile());
			Connection con = myConnection.getConnection();
		
		new XUpdater().execute(con, myConnection.getDatabaseAbbr(), sqlScript,logger);
		}
		else
		{
		if(sqlScript.toLowerCase().indexOf("--freemarker template")>-1)
		{
			/*			boolean isTemplate = getSelect_stmt().toUpperCase().indexOf("FREEMARKER TEMPLATE") > -1;
			String select_string = SqlStringUtils.generateSQL(SxPools.get(mandantenID).getDatabaseAbbr(),
					individualFields.getFormular(), getSelect_stmt() + "\n" + getCleanup_stmt());

			// select_string=select_string.replaceAll("Fächer.elements",
			// "Fächer.xx");
			// map.clear();
			if (isTemplate) {
				SuperXManager.setLastFMMaskenSql(
						"-- " + getMaskInfo() + " " + DateUtils.getNowString() + "\n" + select_string);
				select_string = SxPools.get(mandantenID).getTemplateProcessor().process(map, (Integer) getId(),
						getMaskInfo(), select_string, individualFields, SxPools.get(mandantenID).getRepository(),
						SxPools.get(mandantenID).getSqlDialect());
				// danach enthält map alle vars inkl. sqlvars
			}
			// MB 07/2010 macht Probleme bei Freemarker hashes
			// generell deaktiviert, hier falls doch noch in alten Masken
			// vorhanden
			select_string = SqlStringUtils.removeComment(select_string, "{", "}");
*/
			/* alt: sqlScript=de.superx.bin.FMParser.simpleParser(mySxConnection.getPropfile(), sqlScript);*/
			//Die map und der FieldContainer sind null, werden nur in Masken genutzt
			HashMap map = new HashMap();
			FieldContainer individualFields = new FieldContainer();
			if(SxPools.hasMandanten()){
				//Servlet Betrieb:
				sqlScript= SxPools.get(mandantenId).getTemplateProcessor().process(map, -1,
					"SQL-Script", sqlScript, individualFields, SxPools.get(mandantenId).getRepository(),
					SxPools.get(mandantenId).getSqlDialect());
			}
			else
			{
				//Kommandozeile: TODO hier sind noch keine FM Scripte mit SQLVAR möglich:
				sqlScript=FMParser.simpleParser(mySxConnection.getPropfile(), sqlScript);
			}
			String  keepGenerated=System.getProperty("FreemarkerKeepGenerated");
			if(keepGenerated!=null && keepGenerated.equalsIgnoreCase("true"))
			{
				FileWriter sqlScriptTmp ;
			
				if(sqlScriptFile==null)
				{
					File tempFile = File.createTempFile("fm_output", ".sql");
					sqlScriptTmp = new FileWriter(tempFile);
					logger.info("FM generated SQL: "+tempFile.getAbsolutePath());
				}
				else
					sqlScriptTmp = new FileWriter(sqlScriptFile.getAbsolutePath()+".tmp.sql");
				sqlScriptTmp.write(sqlScript);
				sqlScriptTmp.close();
			}
		
		}
		sqlScript = sqlScript.trim();
		sqlScript = SqlStringUtils.removeComment(sqlScript, "{", "}");
		if (sqlScript.startsWith("\""))
			sqlScript = sqlScript.substring(1, sqlScript.length());
		if (sqlScript.endsWith("\""))
			sqlScript = sqlScript.substring(0, sqlScript.length() - 1);
		//replace CLI-params:
        if (params==null)
        	{
        	//do nothing
        	}
        else
        	 {
    		Enumeration paramEnum = params.propertyNames();
    		while (paramEnum.hasMoreElements()) {
    			String paramName = (String)paramEnum.nextElement();
    			String paramValue = params.getProperty(paramName);
    			sqlScript=de.memtext.util.StringUtils.replace(sqlScript,"$"+paramName, paramValue);
    		}
        }
		logger.config("Inhalt der sql-Datei: "+NEWLINE+"--" + sqlScript + "--");
		SxJdbcClient myClient = new SxJdbcClient(logger, mySxConnection.getPropfile(), sqlScript);
		myClient.Rs_executeALL();
		ResultSet myrs = myClient.getRs();
		if (myrs != null) {
		//Letzter SQL liefert Ergebnisse:
		if(!outfile.equals(""))
		{
			try {
				fw = new FileWriter(outfile);
				if(outFormat.equalsIgnoreCase("xml"))
					fw.write("<?xml version=\"1.0\"  encoding=\""+SqlStringUtils.getEncoding()+"\" ?>");
				myQueryResultSerializer=new QueryResultSerializer("default", mySxConnection, myrs,fw);
				numberOfRows=myQueryResultSerializer.unloadQueryResults("DOQUERY",outFormat,this.CSVdelim,printColNames);
				returnCode=myQueryResultSerializer.getReturnCode();

			} catch (IOException e) {
				logger.severe ("Fehler beim Erstellen der Datei "+outfile);
				returnCode=1;
			}

		}
		else
		{
			sw=new StringWriter();
			if(outFormat.equalsIgnoreCase("xml"))
				sw.write("<?xml version=\"1.0\"  encoding=\"UTF-8\" ?>");
			myQueryResultSerializer=new QueryResultSerializer("default", mySxConnection, myrs,sw);
			numberOfRows=myQueryResultSerializer.unloadQueryResults("DOQUERY",outFormat,this.CSVdelim,printColNames);
			returnCode=myQueryResultSerializer.getReturnCode();
			outString= myQueryResultSerializer.getOutputString().getBuffer();

		}

		

			
		} else
			logger.info("Keine Rückgabewerte aus SQL-Script");
		}
		return returnCode;
	}


	
}
