/*
 * de.superx.etl - a package for controlling ETL routines
 * Copyright (C) 2021 Daniel Quathamer <danielq@memtext.de>
 *
 * This package is licensed under the CampusSource License; 
 * http://www.campussource.de/org/license/
 */
package de.superx.elt.bin;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.StringWriter;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.PreparedStatement; 
import java.sql.SQLException;
import java.sql.Statement;
import java.util.logging.LogManager;
import java.util.logging.Logger;
import java.util.StringTokenizer;
import de.superx.util.SqlStringUtils;
import de.superx.bin.SxConnection;
import de.superx.common.SuperX_el;
import de.superx.elt.QueryResultSerializer;
import de.superx.servlet.SxPools;
import de.superx.servlet.SxSQL_Server;

/**
 * @author Daniel Quathamer Projektgruppe SuperX
 * doquery.java
 * @
 * Dieses Javaprogramm führt einen SQL-Ausdruck aus und gibt das Ergebnis aus.<br>
 * Gebrauch:<br> java doquery <Pfad zu logger-properties> <pfad zu db.properties> <sql-Ausdruck> <Ausgabeformat (txt | html | xml)>(optional) <delimiter> <mit Spaltenüberschriften (true | false)>(optional) <Ausgabedatei>(optional)
 *

 */

public class Doquery {
	private static Connection myDb;
	private static Statement st; // Our statement to run queries with
	private static DatabaseMetaData dbmd;
	// This defines the structure of the database
	private static boolean done = false; // Added by CWJ to permit \q command
	private static String delim = "^";
	private static String header = "false";
	private static String outFormat = "txt";
	private static String logfile = "../conf/logging.properties";
	private static String tabelle = "";
	private static String dbpropfile = "../conf/db.properties";
	private static SxConnection myConnection = null;
	private static String db_driver;
	private static String mandantenID="default";
	private static String outfile = "";
	private static Logger logger =
			(Logger) Logger.getLogger(Doquery.class.toString());
	private static String usage =
			"-------------------------------------\nGebrauch: java de.superx.bin.Doquery $LOGGER_PROPERTIES $DB_PROPERTIES $sql (optional: )$outformat $DBDELIMITER $header $filename  \n---------------------------------------------------";

	public static int go(String args[])
			throws
			Exception {
		if (args.length > 0) {
			logfile = args[0].trim();
		} else {
			throw new IllegalArgumentException("Mindestens drei Parameter (Pfad zu den logger.properties, Pfad zu den db.properties, sql-String) erfoderlich");

		}
		File f = new File(logfile);
		if (!f.exists()) {
			throw new IOException("Datei nicht gefunden: " + logfile);
		}

		FileInputStream ins = new FileInputStream(logfile);
		LogManager MyLogManager = java.util.logging.LogManager.getLogManager();
		MyLogManager.readConfiguration(ins);
		logfile = MyLogManager.getProperty(".level");
		logger.info("Using Loggging-Level " + logfile);
		String query = "";
		long numberOfRows=0;
		int returnCode=0;
		
		if (args.length > 1) {
			dbpropfile = args[1].trim();
		} else {
			logger.severe(
					"Mindestens drei Parameter (Pfad zu den logger.properties, Pfad zu den db.properties, sql-String) erfoderlich");
			System.exit(1);
		}
		if (args.length <= 2) {
			logger.severe(
					"Mindestens drei Parameter (Pfad zu den logger.properties, Pfad zu den db.properties, sql-String) erfoderlich");
			System.exit(1);
		}
		query=args[2].trim();

		if (args.length > 3) {
			outFormat = args[3].trim();
		}

		if (args.length > 4) {
			delim = args[4].trim();
		}
		if (args.length > 5) {
			header = args[5].trim();

		}
		if (args.length > 6) {
			outfile = args[6].trim();

		}
		if (delim.equals(""))
			delim = "^"; //default Delimiter

	
		long jetzt = new java.util.Date().getTime() ;
		getConnection(logger, dbpropfile);
		QueryResultSerializer myQueryResultSerializer = null;
		FileWriter fw;
		StringWriter sw;
		boolean printColname=(header.equalsIgnoreCase("true")?true:false);
		if(!outfile.equals(""))
		{
			try {
				fw = new FileWriter(outfile);
				if(outFormat.equalsIgnoreCase("xml"))
					fw.write("<?xml version=\"1.0\"  encoding=\"UTF-8\" ?>");
				myQueryResultSerializer=new QueryResultSerializer("default", myConnection, query,fw);

			} catch (IOException e) {
				logger.severe ("Fehler beim Erstellen der Datei "+outfile);
				returnCode=1;
			}

		}
		else
		{
			sw=new StringWriter();
			if(outFormat.equalsIgnoreCase("xml"))
				sw.write("<?xml version=\"1.0\"  encoding=\"UTF-8\" ?>");
			myQueryResultSerializer=new QueryResultSerializer("default", myConnection, query,sw);
		}

		numberOfRows=myQueryResultSerializer.unloadQueryResults("DOQUERY",outFormat,delim,printColname);
		returnCode=myQueryResultSerializer.getReturnCode();
		if(outfile.equals(""))
		{
			sw=myQueryResultSerializer.getOutputString();
			System.out.println(sw.toString());
		}
		long erstrecht = new java.util.Date().getTime() ;
		System.out.println( numberOfRows+" rows unloaded  in "+(erstrecht-jetzt)/1000 +" Sec.");
		System.out.println( "returnCode= "+returnCode);
		myQueryResultSerializer = null;
		return returnCode;


	}

	/*
	 * Display some instructions on how to run the example
	 */
	public static void instructions() {
		System.out.println("SuperX @version@\n");
		System.out.println(
				"\nDieses Java-Programm führt einen SQL-Ausdruck aus und gibt das Ergebnis aus.\n");
		System.out.println(
				"Gebrauch:\n java doquery <Pfad zu logger-properties> <pfad zu db.properties> <sql-Ausdruck> <Ausgabeformat (txt | html | xml)>(optional) <delimiter> <mit Spaltenüberschriften (true | false)>(optional) <Ausgabedatei>(optional)\n");
		System.exit(1);
	}

	public static void main(String args[]) {
		int returnCode=0;
		try {
			returnCode=go(args);

		} catch (Exception ex) {
			System.err.println("Doquery Aufruf fehlgeschlagen.\n" + ex);
			ex.printStackTrace();
			System.exit(1);
		}
		System.out.println("Doquery Aufruf ausgeführt.\n" );
		System.exit(returnCode);
	}
	public static void getConnection(Logger logger,String propFile) throws SQLException {
		myConnection = new SxConnection();
		myConnection.setPropfile(propFile);
		logger.config("Starting Connection...");
		try {
			myDb = myConnection.getConnection();
			st = myDb.createStatement();
			//st = myDb.createStatement(java.sql.ResultSet.TYPE_FORWARD_ONLY,
			//          java.sql.ResultSet.CONCUR_READ_ONLY);
			//st.setFetchSize(100); 
			dbmd = myDb.getMetaData();
			//st = myDb.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE,
			//ResultSet.CONCUR_UPDATABLE);
		} catch (Exception e) {
			e.printStackTrace();
			logger.severe("Keine DB-Verbindung: " + e.toString());
			throw new SQLException("Keine DB-Verbindung: " + e.toString());
		}
		db_driver = myConnection.m_DriverClass;

	}


}
