/*
 * de.superx.etl - a package for controlling ETL routines
 * Copyright (C) 2021 Daniel Quathamer <danielq@memtext.de>
 *
 * This package is licensed under the CampusSource License; 
 * http://www.campussource.de/org/license/
 */
package de.superx.elt.bin;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Properties;
import java.util.StringTokenizer;
import java.util.logging.LogManager;
import java.util.logging.Logger;

import de.memtext.util.ExceptionHandler;
import de.memtext.util.PropUtils;
import de.superx.bin.SxConnection;
import de.superx.elt.EtlUtils;
import de.superx.elt.SqlExecutor;
import de.superx.elt.util.GetOpts;

/**
 * @author Daniel Quathamer Projektgruppe SuperX
 * Dosql.java
 * @
 * Dieses Javaprogramm führt einen oder mehrere SQL-Ausdrück in einer Datei aus und gibt das Ergebnis aus.<br>
 * Gebrauch:<br> java dosql <Pfad zu logger-properties> <pfad zu db.properties> <Datei mit sql-Ausdrücken> <Ausgabeformat (txt | html | xml)>(optional) <delimiter> <mit Spaltenüberschriften (true | false)>(optional) <Ausgabedatei>(optional)
 *
 */

/* Änderungen

15.4.2004 MB wenn keine Ausgabedatei und Ausgabeformat txt, alle selects von SxJdbcClient
  			loggen lassen
6.4.2004 MB Code static main verlegt - nach erfolgreicher Durchführung ein system.out 

**/
public class Dosql {
	private static Connection myDb;
	private static DatabaseMetaData dbmd;

	private static Statement st; // Our statement to run queries with
	private static boolean done = false; // Added by CWJ to permit \q command
	private static String delim = "^";
	private static String header = "false";
	private static String logfile = "../conf/logging.properties";
	private static String tabelle = "";
	private static String dbpropfile = "../conf/db.properties";
	private static String outfile = "";
	private static String outFormat = "txt";
	private static String _dateiPfad = "";
	//LogUtils logger=null;
	private static Logger logger =
		(Logger) Logger.getLogger(Dosql.class.toString());
	//static Logger logger = Logger.getLogger(dosql.class);
	private static ExceptionHandler exceptionHandler =
		new ExceptionHandler(false);
	private static SxConnection myConnection = null;
	private static String usage =
		"-------------------------------------\n"
			+ "Gebrauch: java de.superx.bin.Dosql -logger=<<Pfad zu logging.properties>> -dbproperties=<<Pfad zu db.properties>> "
			+ "-sqlfile:<Datei mit sql-Ausdrücken> -params:<Parameter, die in sql- oder Script-Dateien ersetzt werden; Syntax:param1=wert1|param2=wert2 etc> -outFormat:<Ausgabeformat (txt | html | xml)>(optional) -delim:<delimiter> -header:<mit Spaltenüberschriften (true | false)>(optional) -outfile:<Ausgabedatei>(optional) \n---------------------------------------------------";

	/*
	 * Display some instructions on how to run the example
	 */
	public static void instructions() {
		System.out.println("SuperX @version@\n");
		System.out.println(
			"\nDieses Javaprogramm führt ein beliebiges sql-Script mit einer oder mehr sql-Anweisungen aus");
		System.out.println(
			"Im Classpath muss superx@version@.jar sowie der zugehörige jdbc-Treiber sein.");
		System.out.println(
			"Befehl:\n java dosql <Dateipfad sql-Script> <delimiter>(optional) <Ausgabe der Feldüberschriften (optional, true oder false).");
		System.out.println(
			"Default:\n java dosql <Dateipfad sql-Script> <TAB> true");
		System.exit(1);
	}

	public static void main(String args[]) {
		try {
			execute(args);
		} catch (Exception e) {
			logger.severe(
				"Fehler beim sql-Script: " + _dateiPfad + " " + e.toString());
			e.printStackTrace();
			System.exit(1);
		}

	}

	public static void execute(String[] args)
		throws Exception {

		String params = "";
		GetOpts.setOpts(args);
		String isdrin =
			GetOpts.isAllRequiredOptionsPresent(
				"-logger,-dbproperties,-sqlfile");
		if (isdrin != null) {
			System.err.println("Folgende Optionen fehlen: " + isdrin);
			System.err.println(usage);
			System.exit(1);
		}

		//GetOpts myOpts=new GetOpts();
		if (GetOpts.isPresent("-logger"))
			logfile = GetOpts.getValue("-logger");
		if (GetOpts.isPresent("-dbproperties"))
			dbpropfile = GetOpts.getValue("-dbproperties");
		if (GetOpts.isPresent("-sqlfile"))
			_dateiPfad = GetOpts.getValue("-sqlfile");
		if (GetOpts.isPresent("-outFormat"))
			outFormat = GetOpts.getValue("-outFormat");
		if (GetOpts.isPresent("-delim"))
			delim = GetOpts.getValue("-delim");
		if (GetOpts.isPresent("-header"))
			header = GetOpts.getValue("-header");
		if (GetOpts.isPresent("-outfile"))
			outfile = GetOpts.getValue("-outfile");
		if (GetOpts.isPresent("-params"))
			params = GetOpts.getValue("-params");

		if (delim.equals(""))
			delim = "^"; //default Delimiter

		int returnCode=0;
		File f = new File(logfile);
		if (!f.exists()) {
			throw new IOException("Datei nicht gefunden: " + logfile);
		}
		FileInputStream ins = new FileInputStream(logfile);
		LogManager MyLogManager = java.util.logging.LogManager.getLogManager();
		MyLogManager.readConfiguration(ins);
		logfile = MyLogManager.getProperty(".level");
		logger.info("Using Loggging-Level " + logfile);
		File sqlScriptFile = new File(_dateiPfad);
		getConnection(logger, dbpropfile);
		Properties paramProperties=null;
		if(!params.equals(""))
		{
			params=de.memtext.util.StringUtils.replace(params, "|", System.getProperty("line.separator"));
			paramProperties=EtlUtils.convertStringToProperty(params);
		}
		SqlExecutor mySqlExecutor=new SqlExecutor("default", myConnection,sqlScriptFile,paramProperties);
		if(!outfile.equals(""))
			mySqlExecutor.setOutfile(outfile);
		returnCode=mySqlExecutor.executeQueries();
		String output=mySqlExecutor.getOutString().toString();
		
		myConnection.close();

		System.out.println(
			"Dosql hat das Script " + _dateiPfad + " erfolgreich durchgeführt");
		System.out.println(output);
		logger.info("dosql erfolgreich beendet");

	}
	public static void getConnection(Logger logger,String propFile) throws SQLException {
		myConnection = new SxConnection();
		myConnection.setPropfile(propFile);
		logger.config("Starting Connection...");
		try {
			myDb = myConnection.getConnection();
			st = myDb.createStatement();
			//st = myDb.createStatement(java.sql.ResultSet.TYPE_FORWARD_ONLY,
			//          java.sql.ResultSet.CONCUR_READ_ONLY);
			//st.setFetchSize(100); 
			dbmd = myDb.getMetaData();
			//st = myDb.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE,
			//ResultSet.CONCUR_UPDATABLE);
		} catch (Exception e) {
			e.printStackTrace();
			logger.severe("Keine DB-Verbindung: " + e.toString());
			throw new SQLException("Keine DB-Verbindung: " + e.toString());
		}
		

	}
}

