package de.superx.elt.util;

/**
 * Class to analyze arguments passed to Java class
 * Makes ETL compatible to BI 2021.06 and higher
 */
public class GetOptsBackport{
    
    public enum Options{
        opt_logger("-logger"),
        opt_dbprops("-dbproperties"),
        opt_table("-table"),
        opt_outFormat("-outFormat"),
        opt_outFile("-outfile"),
        opt_outfileAttribute("-outfileattribute"),
        opt_cancel("-cancel"),
        opt_sql("-sqlfile"),
        opt_sqlAttribute("-sqlattribute"),
        opt_delim("-delim"),
        opt_header("-header"),
        opt_params("-params"),
        opt_param("-param"),
        opt_in("-in"),
        opt_inFormat("-informat"),
        opt_out("-out"),
        opt_xsl("-xsl"),
        opt_xslPfad("-xslpfad"),
        opt_fopxconf("-fopxconf"),
        opt_user("-user"),
        opt_tid("-tid"),
        opt_mandID("-mandantenid"),
        opt_locale("-locale"),
        opt_simpPars("-simpleParser"),
        opt_webinf("-web_inf_pfad"),
        opt_module("-module_pfad"),
        opt_jobUniqueName("-job_uniquename"),
        opt_uploadpath("-path_to_uploadfile"),
        opt_method("-method"),
        opt_xml("-xml"),
        opt_xmlconfig("-xmlconfig"),
        opt_db("-database"),
        opt_version("-version"),
        opt_unl("-unl"),
        opt_unlParams("-unload_params"),
        opt_jrxml("-jrxml"),
        opt_jasper("-jasper"),
        opt_jrPrint("-jrprint"),
        opt_ignorePagination("-ignore_pagination"),
        opt_idescr("-i_descr"),
        opt_odescr("-o_descr"),
        opt_element("-element"),
        opt_function("-function"),
        opt_mode("-mode"),
        opt_inserts("-inserts"),
        opt_encoding("-encoding"),
        opt_hsnr("-hsnr"),
        opt_noDelete("-nodelete"),
        opt_pause("-pause"),
        opt_jahr("-jahr"),
        opt_datentyp("-datentyp"),
        opt_runs("-runs"),
        opt_checkval("-checkval"),
        opt_url("-url"),
        opt_setname("-setname"),
        
        opt_kernTabellenFreischalten("-kern_tabellen_freischalten"),
        opt_noguiVar0("-nogui"), opt_noguiVar1("-no-gui"), opt_noguiVar2("--no-gui"),
        opt_job("-job");
        
        private String value;
        private Options(String value) {
            this.value = value;
        }
        
        public String stringValue() {
            return value;
        }
    }

    private static String[] arguments = null;
    private GetOptsBackport(){} //Do not instatiate

    /**
     * If you pass the arguments you want to analyse to this static helper class
     * using setOpts, you can later use the short form of the methods e.g.
     * isPresent(String option) without having to pass the arguments again.
     * 
     * @param args
     */
    public static void setOpts(String args[]) {
        arguments = args;
    }

    /**
     * checks if the arguments passed before by setOpts contain the given option
     * 
     * @param Options
     *            option
     * @return true if arguments contain the option, i.e. one String which
     *         starts with the option-String
     */
    public static boolean isPresent(Options option){
        if(arguments == null){
            throw new IllegalStateException("must either call setOpts() before or call the long version of this method");
        }
        return isPresent(arguments, option);
    }

    /**
     * checks if the arguments contain the given option
     * 
     * @param args
     * @param Options
     *            option
     * @return true if arguments contain the option, i.e. one String which
     *         starts with the option-String
     */
    public static boolean isPresent(String args[], Options option){
        boolean result = false;

        for(int i = 0; i < args.length; i++){
            if (args[i] != null && args[i].toLowerCase().startsWith(option.stringValue())){
                result = true;
            }
        }

        return result;
    }

    /**
     * Gets a named option from the arguments passed before with setOps. E.g.
     * getOpt("--log") would return the "--log:true" in the arguments
     * 
     * @param Options
     *            option
     * @return String option and value
     */
    public static String getOpt(Options option){
        if (arguments == null){
            throw new IllegalStateException("must either call setOpts() before or call the long version of this method");
        }
        return getOpt(arguments, option);
    }

    /**
     * Gets a named option from the arguments. E.g. getOpt("--log") would return
     * the "--log:true" in the arguments
     * 
     * @param Options
     *            option
     * @return String option and value
     */
    public static String getOpt(String args[], Options option){
        String result = null;
        for(int i=0; i < args.length; i++){
            if(args[i].toLowerCase().startsWith(option.stringValue())){
                result = args[i];
            }
        }
        return result;
    }

    /**
     * Gets the value of an option from the arguments passed before with setOpts
     * if for example, you ask getValue("--log:") and the arguments passed
     * before with setOpts contain a String "--log:true") "true" is returned
     * 
     * @param Options
     *            option
     * @return String value of the option
     */
    public static String getValue(Options option){
        if (arguments == null){
            throw new IllegalStateException("must either call setOpts() before or call the long version of this method");
        }
        return getValue(arguments, option);
    }

    /**
     * Gets the value of an option, if for example, you ask getValue("--log:")
     * and the arguments passed before with setOpts contain a String
     * "--log:true") "true" is returned
     * 
     * @param Options
     *            option
     * @return String value of the option
     */
    public static String getValue(String args[], Options option) {
        String value = null, temp = getOpt(args, option);
        int pos = option.stringValue().length();
        if(temp.charAt(pos) == ':') {
            pos++;
        }
        value = temp.substring(pos);
        return value;
    }

    /**
     * Prüft, ob alle notwendigen Optionen angegeben sind.
     * 
     * @param Options[]
     *            opts, ein Array mit Optionen die überprüft werden sollen
     * @return null - alles OK, ansonsten String mit den fehlenden Optionen
     */
    public static String isAllRequiredOptionsPresent(Options[] opts){
        String missingOpts = null;
        for (Options o : opts){
            if(!isPresent(o)){
                if(missingOpts == null){
                    missingOpts = o.stringValue()+", ";
                    }else{
                    missingOpts += o.stringValue()+", ";
                }
            }
        }
        return missingOpts;
    }


}
